'''
This script will do checks on the rd_ap_2024 before pushing to feature branch

This problems will be caught before pipelines run
'''
import re

from typing      import Union
from collections import Counter

import argparse
import yaml

from ap_utilities.logging.log_store import LogStore

log = LogStore.add_logger('ap_utilities:check_production')
# --------------------------
class Data:
    '''
    Class storing shared attributes
    '''
    prod_path : str
    regex_info: str = r'"([\w,_,.,-]+)"'
# --------------------------
def _parse_args() -> None:
    parser = argparse.ArgumentParser(description='')
    parser.add_argument('-p', '--prod_path', type=str, help='Path to directory with production, rd_ap_2024', required=True)
    args = parser.parse_args()

    Data.prod_path = args.prod_path
# -------------------------
def _load_yaml(name : str) -> dict:
    path = f'{Data.prod_path}/{name}'
    with open(path, encoding='utf-8') as ifile:
        d_data = yaml.safe_load(ifile)

    return d_data
# -------------------------
def _get_lines(name : str) -> list[str]:
    path = f'{Data.prod_path}/{name}'
    with open(path, encoding='utf-8') as ifile:
        l_line = ifile.read().splitlines()

    return l_line
# -------------------------
def _sample_from_line(line : str) -> Union[None,str]:
    l_match = re.findall(Data.regex_info, line)
    if len(l_match) != 10:
        return None

    sample = l_match[0]

    return sample
# -------------------------
def _samples_from_info_lines( l_line : list[str]) -> list[str]:
    l_sample = [ _sample_from_line(line) for line   in l_line                        ]
    l_sample = [                  sample for sample in l_sample if sample is not None]

    return l_sample
# -------------------------
def _print_repeated(l_line : list[str]) -> None:
    counter = Counter(l_line)
    l_repeated = [ (element, count) for element, count in counter.items() if count > 1 ]

    for repeated, count in l_repeated:
        log.info(f'{repeated:<40}{count:<10}')
# -------------------------
def _list_to_set(l_line : list[str], msg_repeated : Union[None,str]=None) -> set[str]:
    s_line= set(l_line)
    nlist = len(l_line)
    nset  = len(s_line)

    if nlist != nset and msg_repeated is not None:
        log.error('Repeated elements:')
        _print_repeated(l_line)
        raise ValueError(msg_repeated)

    return s_line
# -------------------------
def _print_set(s_data : set[str], msg : Union[str,None] = None) -> None:
    if len(s_data) == 0:
        return

    log.warning(msg)
    for value in s_data:
        log.info(value)
# -------------------------
def _check_mcdt() -> None:
    d_mcdt        = _load_yaml('tupling/config/mcfuntuple.yaml')
    l_mcdt_sample = list(d_mcdt)
    s_mcdt_sample = _list_to_set(l_mcdt_sample, msg_repeated='Found repeated entries in mcfuntuple')


    l_line = _get_lines('info.yaml')
    l_info_sample = _samples_from_info_lines(l_line)
    s_info_sample = _list_to_set(l_info_sample, msg_repeated='Found repeated entries in info.yaml')

    if s_mcdt_sample != s_info_sample:
        log.warning('Samples in mcfuntuple and info.yaml are different')
        s_mcdt_only = s_mcdt_sample - s_info_sample
        s_info_only = s_info_sample - s_mcdt_sample

        _print_set(s_mcdt_only, msg='MCDecayTree only')
        _print_set(s_info_only, msg='info.yaml only')
# -------------------------
def main():
    '''
    Start of execution
    '''
    _parse_args()
    _check_mcdt()
# -------------------------
if __name__ == '__main__':
    main()



'''
Script used to extract decay fields by reading MCDecayTree.py, it uses as input Renato -> DecFiles naming dictionary
'''
import re
import argparse
from typing import Union

import yaml
from ap_utilities.logging.log_store  import LogStore
from ap_utilities.mcdecay.nicknames  import Nicknames

log=LogStore.add_logger('ap_utilities:mcdecay:nicknames')
# ----------------------------
class Data:
    '''
    Class storing shared attributes
    '''
    mode_regex   : str = r'\s+modename\s*=\s*(".*")\s*'
    fields_regex : str = r'\s*FIELDS\s*=\s*{\s*'
    decay_regex  : str = r'\s*"(.*)"\s*:\s*"(.*).*",?'
    d_mode_nick  : dict[str,str]
    samples_path : str
    out_path     : str  = 'decays.yaml'
# ----------------------------
def _get_lines() -> list[str]:
    with open('MCDecayTree.py', encoding='utf-8') as ifile:
        text = ifile.read()

    return text.splitlines()
# ----------------------------
def _get_mode(line : str) -> Union[None,str]:
    mtch = re.match(Data.mode_regex, line)
    if not mtch:
        return None

    mode = mtch.group(1)
    mode = mode.replace('"', '')

    # Needed for lines like:
    # 'Bd_KPi if mode == ':
    if ' ' in mode:
        l_mode = mode.split(' ')
        mode   = l_mode[0]

    return mode
# ----------------------------
def _is_fieldsline(line : str) -> bool:
    mtch = re.match(Data.fields_regex, line)
    if not mtch:
        return False

    return True
# ----------------------------
def _extract_head_decay(line : str) -> Union[None,tuple[str,str]]:
    mtch = re.match(Data.decay_regex, line)

    if not mtch:
        return None

    head  = mtch.group(1)
    decay = mtch.group(2)

    decay = decay.replace(' ', 'space')
    decay = decay.replace('space', ' ')

    return head, decay
# ----------------------------
def _nick_name_from_mode(mode : str) -> str:
    if mode not in Data.d_mode_nick:
        log.warning(f'Cannot find decay {mode}')
        return mode

    return Data.d_mode_nick[mode]
# ----------------------------
def _get_groups(l_line : list[str]) -> dict[str,dict[str,str]]:
    save = False

    d_group : dict[str,dict[str,str]] = {}
    this_mode = 'no_mode'
    for line in l_line:
        mode_name = _get_mode(line)
        if mode_name is not None:
            this_mode = mode_name
            d_group[this_mode] = {}

        save = save or _is_fieldsline(line)
        if save and 'FIELDS' not in line:
            val = _extract_head_decay(line)
            if val is None:
                continue

            head, decay = val
            head = head.ljust(15)
            d_group[this_mode][head] = decay

        if save and '}' in line:
            save = False

    d_group_trimm = { _nick_name_from_mode(mode) : d_decay for mode, d_decay in d_group.items() if len(d_decay) > 0 }

    return d_group_trimm
# ----------------------------
def _format_line(line : str) -> str:
    nquote = line.count('\'')
    if nquote != 4:
        return line

    line = line.replace('\'', '', 2)

    return line
# ----------------------------
def _post_process(path : str) -> None:
    with open(path, encoding='utf-8') as ifile:
        l_line = ifile.read().splitlines()

    l_line_form = [ _format_line(line) for line in l_line ]

    form_path = path.replace('.yaml', '_form.yaml')
    with open(form_path, 'w', encoding='utf-8') as ofile:
        text = '\n'.join(l_line_form)
        ofile.write(text)
# ----------------------------
def _parse_args() -> None:
    parser = argparse.ArgumentParser(description='This script will pick up the info.yaml with the old naming and produce a YAML with a dictionary from old sample naming to new one')
    parser.add_argument('-s','--samples', type=str, help='Path to info.yaml with old naming', required=True)
    args = parser.parse_args()

    Data.samples_path = args.samples
# ----------------------------
def main():
    '''
    Script starts here
    '''
    _parse_args()

    obj = Nicknames(Data.samples_path)
    Data.d_mode_nick = obj.get_nicknames()

    l_line = _get_lines()
    d_group= _get_groups(l_line)

    log.info(f'Saving decays list to: {Data.out_path}')
    with open(Data.out_path, 'w', encoding='utf-8') as ofile:
        yaml.safe_dump(d_group, ofile, width=200)

    _post_process(Data.out_path)
# ----------------------------
if __name__ == '__main__':
    main()



'''
Module with functions used to test functions in physics/utilities.py
'''
import pytest

import ap_utilities.decays.utilities as aput

# --------------------------------------------------
class Data:
    '''
    Class used to store data needed by tests
    '''

    l_event_type = [
            '10000000',
            '10000010',
            '10000020',
            '10000021',
            '10000022',
            '10000023',
            '10000027',
            '10000030',
            '10002203',
            '10002213',
            '11100001',
            '11100003',
            '11100006',
            ]
# --------------------------------------------------
@pytest.mark.parametrize('event_type', Data.l_event_type)
def test_read_decay_name(event_type : str) -> None:
    '''
    Tests reading of decay name from YAML using event type
    '''
    literal = aput.read_decay_name(event_type=event_type, style='literal')
    safe_1  = aput.read_decay_name(event_type=event_type, style= 'safe_1')

    print(f'{literal:<50}{safe_1:<50}')
